package pw.stadnik.common;


import java.io.Serializable;
import java.net.URLDecoder;

/**
 * This class represents the single HTML Tag A element 
 * which contains elements associated with this tag.
 *
 * @author Jakub Stadnik
 *
 */
public class HTMLTagA implements Serializable, Cloneable
{
	/**
	 * Serial version UID. It is used when 
	 * serializing a particular object of this class.
	 */
	private static final long serialVersionUID = 513577004080746588L;
	/**
	 * HREF attribute of the tag
	 */
	private String HREF;
	/**
	 * Value of the tag
	 */
	private String linkName;
	
	/**
	 * Rank of the tag. Interpretation of it 
	 * is algorithm dependent.
	 */
	private Float rank = 0.0f;
	
	/**
	 * Returns rank
	 * 
	 * @return rank of the tag
	 */
	public Float getRank()
	{
		return rank;
	}
	/**
	 * Sets rank
	 * 
	 * @param rank new rank of the tag
	 */
	public void setRank(Float rank)
	{
		this.rank = rank;
	}
	
	/**
	 * Engine that returned the link.
	 */
	private String engineName;
	
	public static long getSerialVersionUID()
	{
		return serialVersionUID;
	}
	/**
	 * Returns HREF
	 * 
	 * @return href attribute of the tag
	 */
	public String getHREF()
	{
		return HREF;
	}
	/**
	 * Returns linkName
	 * 
	 * @return value of the tag
	 */
	public String getLinkName()
	{
		return linkName;
	}
	/**
	 * Sets HREF
	 * 
	 * @param href href attribute of the tag
	 */
	public void setHREF(String href)
	{
		HREF = href;
		try
		{
			//this.url = new URL(href);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	/**
	 * Sets linkName
	 * 
	 * @param linkName value of the tag
	 */
	public void setLinkName(String linkName)
	{
		this.linkName = linkName;
	}
	/**
	 * Returns string representation of this HTMLTagA
	 * object. The string representation is
	 * &lt;linkName&gt;;&lt;HREF&gt;.
	 * 
	 **/
	public String toString()
	{
		String url = "";
		try
		{
			url = URLDecoder.decode(this.HREF, "UTF-8");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			url = this.HREF;
		}
		return this.linkName + ";" + url;
	}
	
	/**
	 * Default constructor.
	 */
	public HTMLTagA()
	{
		
	}
	
	/**
	 * Constructor initializing fields.
	 * 
	 * @param href href attribute of new HTMLTagA object
	 * @param linkName value of the new HTMLTagA object
	 */
	public HTMLTagA(String href, String linkName)
	{
		this.HREF = href;
		this.linkName = linkName;
	}
	
	/**
	 * Constructor initializing fields.
	 * 
	 * @param href href attribute of new HTMLTagA object
	 * @param linkName value of the new HTMLTagA object
	 * @param rank rank of the new HTMLTagA object
	 */
	public HTMLTagA(String href, String linkName, Float rank)
	{
		this.HREF = href;
		this.linkName = linkName;
		this.rank = rank;
		try
		{
			//this.url = new URL(href);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	/** 
	 * Method tests if two HTMLTagA objects are equal.
	 * Equality of those two objects means that one's
	 * HREF attribute points in contained in other's HREF
	 * or vice-versa.
	 * 
	 * @return true if two HREFs are equal in aforementioned sense
	 **/
	@Override
	public boolean equals(Object o)
	{
		if(!(o instanceof HTMLTagA))
			return false;
		HTMLTagA tagA = (HTMLTagA)o;
		try
		{
			//return url.sameFile(tagA.url);
			String thisHREF = this.HREF.toLowerCase().replace("http://", "").replace("https://", "").replace("www.", "");
			String otherHREF = tagA.HREF.toLowerCase().replace("http://", "").replace("https://", "").replace("www.", "");
			if(thisHREF.equals(otherHREF))// || otherHREF.contains(thisHREF))
				return true;
			else
				return false;
			//return this.HREF.equalsIgnoreCase(tagA.HREF);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return false;
		
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone()
	{
		try
		{
			HTMLTagA tagA = (HTMLTagA)super.clone();
			tagA.HREF = new String(this.HREF);
			tagA.linkName = new String(this.linkName);
			tagA.rank = new Float(this.rank);
			tagA.engineName = new String(this.engineName);
			return tagA;
		}
		catch(CloneNotSupportedException e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	
	/**
	 * Returns engine name
	 * 
	 * @return engine name
	 */
	public String getEngineName()
	{
		return engineName;
	}
	/**
	 * Sets engine name
	 * 
	 * @param engineName engineName to associate with this HTMLTagA object
	 */
	public void setEngineName(String engineName)
	{
		this.engineName = engineName;
	}
	
}
