package pw.stadnik.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * This class is used to transfer initial search parameters 
 * from the client to the main application. It is also used to 
 * transfer answers from the main application to the client. 
 * 
 * @author Jakub Stadnik
 *
 */
public class SearchParams implements Serializable
{
	/**
	 * Serial version UID. It is used when 
	 * serializing a particular object of this class.
	 */
	private static final long serialVersionUID = -8947215741550612766L;
	/**
	 * Initial query that is input by the user. It can 
	 * contain more than one word that is why it is a 
	 * <code>List&ltString&rt</code> of words.
	 */
	private List<String> phrases;
	/**
	 * Name of the algorithm that will be used
	 * during the negotiation process.
	 */
	private String algorithm;
	
	/**
	 * Returns phrases
	 * 
	 * @return list of phrases
	 */
	public List<String> getPhrases()
	{
		return phrases;
	}
	/**
	 * Sets phrases
	 * 
	 * @param phrases list of phrases
	 */
	public void setPhrases(ArrayList<String> phrases)
	{
		this.phrases = phrases;
	}
	/**
	 * Returns algorithm
	 *  
	 * @return name of the algorithm
	 */
	public String getAlgorithm()
	{
		return algorithm;
	}
	/**
	 * Sets algorithm
	 * 
	 * @param algorithm name of the algorithm
	 */
	public void setAlgorithm(String algorithm)
	{
		this.algorithm = algorithm;
	}
	
	
}
