package pw.stadnik.searcher;

import java.sql.Connection;
import java.sql.DriverManager;


/**
 * DataAccess is an abstract class,
 * providing access to the database.
 * Any class that will provide the database connection
 * for any module should extend this class.
 * 
 * @author Jakub Stadnik
 *
 */
public abstract class DataAccess
{

	/**
	 * Database URL
	 */
	protected String url = "jdbc:mysql://127.0.0.1:3306/searcher?jdbcCompliantTruncation";
	
	/**
	 * Database user
	 */
	protected String user = "root";
	
	/**
	 * Database password
	 */
	protected String password = "";
	
	/**
	 * Connection object
	 */
	protected Connection connection = null;
	
	/**
	 * Method opens new database connection.
	 */
	public void openConnection()
	{
		try
		{
			Class.forName("com.mysql.jdbc.Driver");
			connection = DriverManager.getConnection(url, user, password);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Method closes the connection to the
	 * database.
	 */
	public void closeConnection()
	{
		try
		{
			connection.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	public void finalize()
	{
		this.closeConnection();
	}
}
