package pw.stadnik.searcher;

import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;

import org.htmlparser.Node;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.filters.AndFilter;
import org.htmlparser.filters.NodeClassFilter;
import org.htmlparser.sax.Attributes;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;
import org.xml.sax.helpers.NamespaceSupport;

import pw.stadnik.common.HTMLTagA;

/**
 * LinkExtractor extracts all the links from the given webpage.
 * 
 * @author Jakub Stadnik
 */
public class LinkExtractor
{
    /**
     * 
     * Run the link extractor. 
     * 
     * @param connection connection from which the links shall be extracted
     * 
     * @return list of extracted links
     */
    public List<HTMLTagA> extractLinks (URLConnection connection)
    {
        Parser parser;
        NodeFilter filter;
        NodeList list  = new NodeList();
        List<HTMLTagA> links = new ArrayList<HTMLTagA>();

        filter = new NodeClassFilter (LinkTag.class);
        filter = new AndFilter (filter,
                new NodeFilter ()
                {
                    /**
					 * 
					 */
					private static final long serialVersionUID = -3403742842894162180L;

					public boolean accept (Node node)
                    {
                        return (((LinkTag)node).isHTTPLikeLink());
                    }
                }
            );
        try
        {
            parser = new Parser (connection);
            list = parser.extractAllNodesThatMatch (filter);
            for (int i = 0; i < list.size (); i++)
            {
            	LinkTag lt = (LinkTag)list.elementAt(i);
            	HTMLTagA link = new HTMLTagA();
            	link.setLinkName(extractLinkName(list.elementAt(i).toHtml()));
            	Attributes attribs = new Attributes(lt, new NamespaceSupport(), new String[10]);
            	String href = attribs.getValue("href");
            	link.setHREF(href);
            	links.add(link);
                //System.out.println ();
            }
        }
        catch (ParserException e)
        {
            e.printStackTrace ();
        }
        return links;
    }
    
    /**
     * Method extracts the link value
     * from the tag. Link value is the
     * part between <code>&lt;a&gt;</code> and
     * <code>&lt;/a&gt;</code>.
     * 
     * @param link whole tag from which the name should be extracted
     * @return value of the link
     */
    private String extractLinkName(String link)
    {
    	String linkName = "";
    	int openTagIndex = link.indexOf(">");
    	linkName = link.substring(openTagIndex + 1);
    	//linkName = linkName.replace("<b>", "").replace("</b>", "").replace("<u>", "").replace("</u>", "");
    	int closeTagIndex = linkName.indexOf("</a>");
    	if(closeTagIndex > -1)
    		linkName = linkName.substring(0, closeTagIndex);
    	return linkName;
    }
    
   /* private String extractHREF(String link)
    {
    	String HREF = "";
    	link = link.replace("href=", "href=\"");
    	int hrefIndex = link.toLowerCase().indexOf("href=");
    	HREF = link.substring(hrefIndex + 6);
    	int indexOfQuote = HREF.indexOf("\"");
    	if(indexOfQuote > 0)
    		HREF = HREF.substring(0, indexOfQuote);
    	
    	return HREF;
    }*/
    
}