package pw.stadnik.searcher;

import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;

import pw.stadnik.common.HTMLTagA;
import pw.stadnik.searcher.searchengine.SearchEngine;

/**
 * 
 * This class utilizes parser of the web pages
 * returned by any search engine. It also 
 * filters any links that should not be
 * treated as results.
 * 
 * @author Jakub Stadnik
 *
 */
public class SearchEngineExplorer
{

	/**
	 * An URL which will be used to extract answers from.
	 */
	private URL query = null;

	/**
	 * List of temporary HREFs
	 */
	private List<String> HREFs = new ArrayList<String>();

	/**
	 * Number of how many results should be returned
	 */
	private Integer howManyToGet = 0;
	/**
	 * List of extracted results
	 */
	private List<HTMLTagA> results = new ArrayList<HTMLTagA>();
	
	/**
	 * Searchengine object used to extract the results from
	 */
	private SearchEngine searchEngine;

	/**
	 * Constructor initializing necessary fields.
	 * 
	 * @param searchEngine searchEngine that is to be used during extraction process
	 * @param query query that is to be supplied to the search engine
	 * @param howManyToGet how many results should the search engine return
	 */
	public SearchEngineExplorer(
			SearchEngine searchEngine, URL query, int howManyToGet)
	{
		this.searchEngine = searchEngine;
		this.query = query;
		this.howManyToGet = howManyToGet;
	}

	/**
	 * This method parses the given URL finding HREFs 
	 * and their descriptions (link names).
	 **/
	private void beginExploring()
	{
		try
		{
			URL url = query;
			URLConnection connection = url.openConnection();
			connection.setRequestProperty("User-Agent", "Mozilla");
			if(searchEngine.getCookie() != null)
				connection.setRequestProperty("Cookie", searchEngine.getCookie());
			LinkExtractor le = new LinkExtractor();
			System.out.println("Link extractor starting");
			System.out.println("URL:" + url.toString());
			List<HTMLTagA> linkList = le.extractLinks(connection);
			System.out.println("Links extracted");
			List<HTMLTagA> ignoreLinks = searchEngine.getIgnoreLinks();
			for(int i = 0; i < linkList.size(); i++ )
			{
			  
				HTMLTagA link = linkList.get(i);
				String linkName = link.getLinkName();
				String href = link.getHREF();
				if(href == null || linkName == null)
					continue;
					
				boolean isIgnored = false;
				
				for(int j = 0; j < ignoreLinks.size(); j++)
				{
					HTMLTagA ignore = ignoreLinks.get(j);
					if(linkName.contains(ignore.getLinkName()) && ignore.getHREF().equals(""))
						isIgnored = true;
					else if(href.contains(ignore.getHREF()) && ignore.getLinkName().equals(""))
						isIgnored = true;
					else if(linkName.contains(ignore.getLinkName()) &&
							href.contains(ignore.getHREF()))
						isIgnored = true;
					
				}
				
				if (!isIgnored)
				{
					if(!HREFs.contains(href))
					{
						HREFs.add(href);
						HTMLTagA tagA = new HTMLTagA();
						tagA.setHREF(href);
						tagA.setLinkName(linkName);
						tagA.setEngineName(this.searchEngine.getName());
						results.add(tagA);
					}
				}
				/*(else
				{
					HTMLTagA sei = new HTMLTagA();
					sei.setHREF(href);
					sei.setLinkName(linkName);
					collectedLinks.add(sei);
				}*/
			}
			while(results.size() > this.howManyToGet)
			{
				results.remove(results.size() - 1);
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			System.out
					.println("Exception: DomainExplorer: Agent has failed!!! "
							+ searchEngine.getName());
		}

	}

	/**
	 * This method starts the URL parser.
	 * Afterwards it returns the list of results.
	 * 
	 * @return list of results
	 */
	public List<HTMLTagA> getResults()
	{
		beginExploring();
		return results;
	}
}

