package pw.stadnik.searcher.agent;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import pw.stadnik.common.HTMLTagA;
import pw.stadnik.searcher.SearchEngineExplorer;
import pw.stadnik.searcher.algorithms.MethodType;
import pw.stadnik.searcher.searchengine.SearchEngine;

/**
 * Search agent is the agent responsible
 * for querying the search engines for answers.
 * 
 * @author Jakub Stadnik
 *
 */
//@SuppressWarnings("unused")
public class SearchAgent extends Agent
{

	/**
	 * Name of the agent
	 */
	private String name;
	/**
	 * Selected engine after the negotiation
	 * process.
	 */
	private SearchEngine selectedEngine = null;
	/**
	 * Identifier of manager agent.
	 */
	private AID managerAgentId = null;
	/**
	 * Query with which the search
	 * engines will queried.
	 */
	private List<String> phrases = null;
	/**
	 * Number of results that each search engine should return
	 */
	private Integer resultCount;
	/**
	 * Selected algorithm of answer processing.
	 */
	private MethodType selectedMethod = null;
	/**
	 * Weight that is received after
	 * it is calculated by search engine.
	 */
	private Float weight = null;
	/**
	 * Serial version UID. It is used when 
	 * serializing a particular object of this class.
	 */
	private static final long serialVersionUID = 7154457625153330077L;
	
	/**
	 * Method puts agent into the waiting state.
	 * Agent waits for the algorithm that will be used.
	 */
	private void obtainAlgorithm()
	{
		ACLMessage message = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
		this.selectedMethod = MethodType.valueOf(message.getContent());
	}
	
	/**
	 * Method performs search for answers 
	 * using selected search engine
	 * by utilizing search engine explorer
	 * which in turn transforms the resulting web site
	 * into the list of answers.
	 */
	private void performSearch()
	{
		//SearchEngineDataAccess dataAccess = new SearchEngineDataAccess();
		/*dataAccess.openConnection();
		ArrayList<SearchEngine> engines = dataAccess.getEngines();
		SearchEngine se = new SearchEngine();
		for(int i = 0; i < engines.size(); i++)
		{
			if(engines.get(i).getName().equals(searchEngineName))
				se = engines.get(i);
		}
		se.getIgnoreData();
		dataAccess.closeConnection();
		ArrayList<String> phrases = new ArrayList<String>();
		phrases.add("farmetrics");*/
		//phrases.add("agents");
		System.out.println(this.name + ": Search starting");
		if(this.selectedEngine == null)
			return;
		URL url = null;
		try
		{
			url = selectedEngine.constructQuery(phrases, resultCount);
			//System.out.print(url);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		//searchEngine.getIgnoreData();
		System.out.println(this.name + " Query constructed");
		SearchEngineExplorer explorer = new SearchEngineExplorer(selectedEngine, url, resultCount);
		System.out.println(this.name + " Explorer created");
		List<HTMLTagA> results = explorer.getResults();
		System.out.println(this.name + ": search finished");
		ACLMessage resultMessage = new ACLMessage(ACLMessage.PROPOSE);
		try
		{
			resultMessage.setContentObject((ArrayList<HTMLTagA>)results);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		resultMessage.addReceiver(managerAgentId);
		send(resultMessage);
		//return;// explorer.getResults();
	}
	
	/**
	 * Method returns search engines
	 * in order depending on knowledge base
	 * entries. If the knowledge base is empty
	 * the order is random.
	 */
	/*private void getSearchEngines()
	{
		AgentDataAccess dataAccess = new AgentDataAccess();
		dataAccess.openConnection();
		
		String query = phrases.get(0);
		for(int i = 1; i < phrases.size(); i++)
		{
			query = query + " " + phrases.get(i);
		}
		
		searchEngineIds = dataAccess.getSearchEngines(query, this.selectedMethod.toString());
		
		dataAccess.closeConnection();
	}*/
	
	/**
	 * Method puts the agent into the waiting state.
	 * Agent waits for the weight to be received
	 * from manager agent.
	 */
	private void obtainWeight()
	{
		ACLMessage weight = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
		try
		{
			this.weight = (Float)weight.getContentObject();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Process of negotiation of search engine
	 * with manager agent. Agent will ask for
	 * engine according to the order those
	 * were received from the database.
	 * If a particular engine is taken,
	 * an agent will try next and next until
	 * an engine is selected.
	 * 
	 */
	/*private void negotiateSearchEngine()
	{
		while(selectedEngine == null)
		for(int i = 0; i < searchEngineIds.size(); i++)
		{
			ACLMessage engineRequest = new ACLMessage(ACLMessage.REQUEST);
			engineRequest.setContent(searchEngineIds.get(i).toString());
			System.out.println(this.name + " wants engine: " + searchEngineIds.get(i).toString());
			engineRequest.addReceiver(managerAgentId);
			send(engineRequest);
			ACLMessage managerResponse = blockingReceive();
			if(managerResponse.getPerformative() == ACLMessage.AGREE)
			{
				try
				{
					selectedEngine = (SearchEngine)managerResponse.getContentObject();
					if(selectedEngine == null)
					{
						ACLMessage wrongEngine = new ACLMessage(ACLMessage.INFORM);
						wrongEngine.setContent("NOT OK");
						wrongEngine.addReceiver(managerAgentId);
						send(wrongEngine);
						doDelete();
						break;
					}
					ACLMessage confirmation = new ACLMessage(ACLMessage.INFORM);
					confirmation.setContent("OK");
					confirmation.addReceiver(managerAgentId);
					System.out.println(this.name + " Engine: " + selectedEngine.getName());
					send(confirmation);
					break;
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}
	}*/
	
	/**
	 * Method puts agent into the waiting state.
	 * Agents wait for the query that is to be received
	 * from manager agent.
	 */
	@SuppressWarnings("unchecked")
	private void obtainQuery()
	{
		ACLMessage msg = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
		try
		{
			phrases = (ArrayList<String>)msg.getContentObject();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		msg = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
		resultCount = Integer.parseInt(msg.getContent());
	}
	
	/**
	 * Method updates the knowledge base
	 * according to the results of the algorithms 
	 * and/or user feedback.
	 */
	private void updateKnowledgeBase()
	{
		AgentDataAccess dataAccess = new AgentDataAccess();
		dataAccess.openConnection();
		String query = "";
		query = phrases.get(0);
		for(int i = 1; i < phrases.size(); i++)
		{
			query = query + " " + phrases.get(i);
		}
		dataAccess.updateKnowledgeBase(query, this.selectedEngine.getId(), this.weight, this.selectedMethod.toString());
		dataAccess.closeConnection();
	}
	
	/**
	 * Method wraps all other methods
	 * needed for query processing.
	 */
	private void processQuery()
	{
		
		obtainQuery();
		obtainAlgorithm();
		//getSearchEngines();
		//negotiateSearchEngine();
		performSearch();
		if(!(selectedMethod == MethodType.STATISTICS))
		{
			obtainWeight();
			updateKnowledgeBase();
		}
	}
	
	/**
	 * Method puts agent into waiting state. In it 
	 * agent waits for SearchEngine object to be received
	 * from ManagerAgent.
	 */
	private void getSearchEngine()
	{
		try
		{
			ACLMessage engineMessage = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.PROPAGATE));
			this.selectedEngine = (SearchEngine)engineMessage.getContentObject();
			managerAgentId = engineMessage.getSender();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
	}

	/* (non-Javadoc)
	 * @see jade.core.Agent#setup()
	 */
	public void setup()
	{
		Object[] args = getArguments();
		//this.id = (Integer)args[0];
		this.name = (String)args[1];
		System.out.println("Agent " + getAID() + " started");
		getSearchEngine();
		addBehaviour(new CyclicBehaviour(this)
		{

			/**
			 * 
			 */
			private static final long serialVersionUID = 6237029653653881282L;
			@Override
			public void action()
			{	
				//System.out.println("Waiting for request");
				processQuery();
			}
			
			public int onEnd() 
			{
				myAgent.doDelete();
				return super.onEnd();
		    } 
		});
	}

}
