package pw.stadnik.searcher.searchengine;

import java.sql.*;
import java.util.ArrayList;

import pw.stadnik.searcher.DataAccess;
import pw.stadnik.common.HTMLTagA;

/**
 * Class provides database access to search engine
 * functionalities.
 * 
 * @author Jakub Stadnik
 *
 */
public class SearchEngineDataAccess extends DataAccess
{
	/**
	 * Method retrieves the configured search engines 
	 * from the database.
	 * 
	 * @return list of search engine objects
	 */
	public ArrayList<SearchEngine> getEngines()
	{
		ArrayList<SearchEngine> searchEngines = new ArrayList<SearchEngine>();
		try
		{
			String sql = "SELECT id, name, query_parameter, result_count_parameter, cookie_based_parameters FROM search_engine WHERE is_enabled = 1";
			
			Statement statement = connection.createStatement();
			
			ResultSet resultSet = statement.executeQuery(sql);
			
			while(resultSet.next())
			{
				SearchEngine engine = new SearchEngine();
				engine.setId(resultSet.getInt("id"));
				engine.setName(resultSet.getString("name"));
				engine.setQueryParameter(resultSet.getString("query_parameter"));
				engine.setResultCountParameter(resultSet.getString("result_count_parameter"));
				engine.setCookieBasedParam(resultSet.getString("cookie_based_parameters"));
				searchEngines.add(engine);
			}
			
			statement.close();
			resultSet.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return searchEngines;
	}
	
	/**
	 * Method gets the list of links
	 * that should be ignored for a given search engine.
	 * 
	 * @param searchEngineId search engine database identifier
	 * @return list of links
	 */
	public ArrayList<HTMLTagA> getIgnoreData(Integer searchEngineId)
	{
		ArrayList<HTMLTagA> ignoreData = new ArrayList<HTMLTagA>();
		try
		{
			String sql = "SELECT href, link_name FROM search_engine_ignore WHERE search_engine_id = " + searchEngineId + " OR search_engine_id IS NULL";
			
			Statement statement = connection.createStatement();
			
			ResultSet resultSet = statement.executeQuery(sql);
			
			while(resultSet.next())
			{
				HTMLTagA ignore = new HTMLTagA();
				ignore.setHREF(resultSet.getString("href"));
				ignore.setLinkName(resultSet.getString("link_name"));
				ignoreData.add(ignore);
			}
			resultSet.close();
			statement.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return ignoreData;
	}
}
