package pw.stadnik.searcher.unused;

import java.io.File;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.hssf.util.Region;

import pw.stadnik.common.HTMLTagA;
import pw.stadnik.searcher.algorithms.AlgorithmHelper;

/**
 * 
 * Class used for statistics generation.
 * Contains all necessary methods for statistics generation.
 * 
 * @author Jakub Stadnik
 *
 */
public class Statistics
{
	private HSSFWorkbook workbook = null;
	
	@SuppressWarnings("unchecked")
	public void generateReport(OutputStream out, String query, Map<String, List<HTMLTagA>> results)
	{
		try
		{
			workbook = new HSSFWorkbook();
			File f = new File("Statistics for " + query + ".xls");
			if(f.exists())
				f.delete();
			HSSFSheet sheet = workbook.createSheet("Statistics for " + query);
			
			createSummarySheetHeaders();
			
			Map<String, List<HTMLTagA>> trimmedResults = new HashMap<String, List<HTMLTagA>>();
			
			int minSize = Integer.MAX_VALUE;
			
			Map<String, List<HTMLTagA>> clonedResults = (Map<String, List<HTMLTagA>>)((HashMap<String, List<HTMLTagA>>)results).clone();
			
			List<String> emptyResults = new ArrayList<String>();
			
			for(String s : results.keySet())
			{
				if(results.get(s).size() == 0)
				{
					clonedResults.remove(s);
					emptyResults.add(s);
				}
			}
			
			results = clonedResults;
			
			for(String s : results.keySet())
			{
				if(minSize > results.get(s).size())
				{
					minSize = results.get(s).size();
				}
			}
			
			for(String s : results.keySet())
			{
				List<HTMLTagA> links = results.get(s);
				List<HTMLTagA> clonedLinks = new ArrayList<HTMLTagA>();
				
				for(int i = 0; i < minSize; i++)
				{
					HTMLTagA tagA = (HTMLTagA)links.get(i).clone();
					clonedLinks.add(tagA);
				}
				trimmedResults.put(s, clonedLinks);
			}
			
			Map<HTMLTagA, LinkProperties> linkProperties = generateStatistics(trimmedResults);
				
			int row = 8;
			
			HSSFCell averageCell = sheet.createRow(3).createCell((short)0);
			HSSFCell stdDevCell = sheet.createRow(3).createCell((short)1);
			
			averageCell.setCellValue(calculateAverage(trimmedResults));
			stdDevCell.setCellValue(Math.sqrt(calculateVariance(trimmedResults)));
			
			Set<Entry<HTMLTagA, LinkProperties>> entriesSet = linkProperties.entrySet();
			
			Entry<HTMLTagA, LinkProperties>[] entriesArray = (Entry<HTMLTagA, LinkProperties>[])new Entry[linkProperties.size()];
			
			entriesArray = entriesSet.toArray(entriesArray);
			
			Arrays.sort(entriesArray, new Comparator<Entry<HTMLTagA, LinkProperties>>()
					{

						@Override
						public int compare(Entry<HTMLTagA, LinkProperties> o1,
								Entry<HTMLTagA, LinkProperties> o2)
						{
							return o1.getValue().getNumberOfOccurences().compareTo(o2.getValue().getNumberOfOccurences()) * -1;
						}
					
					});
			
			for(int i = 0; i < entriesArray.length; i++)
			{
				Entry<HTMLTagA, LinkProperties> entry = entriesArray[i];
				
				HSSFRow rowi = sheet.createRow(row);
				
				HSSFCell cell0 = rowi.createCell((short)0);
				HSSFCell cell1 = rowi.createCell((short)1);
				HSSFCell cell2 = rowi.createCell((short)2);
				HSSFCell cell3 = rowi.createCell((short)3);
				HSSFCell cell4 = rowi.createCell((short)4);
				HSSFCell cell5 = rowi.createCell((short)5);
				HSSFCell cell6 = rowi.createCell((short)6);
				HSSFCell cell7 = rowi.createCell((short)7);
				
				cell0.setCellValue(new HSSFRichTextString(entry.getKey().getHREF()));
				cell1.setCellValue(new HSSFRichTextString(entry.getValue().getSearchEngine()));
				cell2.setCellValue(entry.getValue().getNumberOfOccurences());
				cell3.setCellValue(entry.getValue().getAveragePosition());
				cell4.setCellValue(entry.getValue().getMinimalPosition());
				cell5.setCellValue(entry.getValue().getMaximalPosition());
				
				Integer samePlaceOccurences = 0;
				if(entry.getValue().getNumberOfSameplaces() != null)
				{
					samePlaceOccurences = entry.getValue().getNumberOfSameplaces();
				}
				cell6.setCellValue(samePlaceOccurences);
				
				Map<Integer, Integer> givenPlaceOccurences = entry.getValue().getNumberOfOccurencesAtGivenPlace();
				Map<Integer, String> givenPlaceEngines = entry.getValue().getEnginesHavingAtGivenPlace();
				
				String s = "";
				
				for(Integer place : givenPlaceOccurences.keySet())
				{
					s+= place + "--" + givenPlaceOccurences.get(place) + "( " + givenPlaceEngines.get(place) + ");";
				}
				
				cell7.setCellValue(new HSSFRichTextString(s));
				
				row++;
			}
			
			
			
			for(String s : trimmedResults.keySet())
			{
				workbook.createSheet(s);
				fillEngineSheet(s, trimmedResults.get(s), trimmedResults);
			}
			
			for(short i = 0; i < 6; i++)
			{
				sheet.setColumnWidth(i, (short)8000);
			}
			
			workbook.write(out);
			out.close();
			workbook = null;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private Map<HTMLTagA, LinkProperties> generateStatistics(Map<String, List<HTMLTagA>> results)
	{
		Map<HTMLTagA, LinkProperties> linkProperties = new HashMap<HTMLTagA, LinkProperties>();
		
		List<HTMLTagA> tags = new ArrayList<HTMLTagA>();
		List<LinkProperties> properties = new ArrayList<LinkProperties>();
		
		for(String s : results.keySet())
		{
			List<HTMLTagA> links = results.get(s);
			
			for(int i = 0; i < links.size(); i++)
			{
				HTMLTagA tagA = links.get(i);
				LinkProperties linkProps = null;
				if(!tags.contains(tagA))
				{
					tags.add(tagA);
					linkProps = new LinkProperties();
					linkProps.setSearchEngine(s);
					linkProps.setNumberOfOccurences(1);
					linkProps.setNumberOfSameplaces(0);
					linkProps.getNumberOfOccurencesAtGivenPlace().put(i + 1, 1);
					linkProps.getEnginesHavingAtGivenPlace().put(i + 1, s);
					linkProps.setAveragePosition((float)(i + 1));
					linkProps.setMaximalPosition(i + 1);
					linkProps.setMinimalPosition(i + 1);
					properties.add(linkProps);
				}
				else
				{
					int index = tags.indexOf(tagA);
					linkProps = properties.get(index);
					linkProps.setSearchEngine(linkProps.getSearchEngine() + "," + s);
					linkProps.setNumberOfOccurences(linkProps.getNumberOfOccurences() + 1);
					linkProps.setAveragePosition(linkProps.getAveragePosition() + (float)(i + 1));
					linkProps.setMaximalPosition(Math.max(i + 1, linkProps.getMaximalPosition()));
					linkProps.setMinimalPosition(Math.min(i + 1, linkProps.getMinimalPosition()));
					if(linkProps.getNumberOfOccurencesAtGivenPlace().containsKey(i + 1))
					{
						linkProps.getNumberOfOccurencesAtGivenPlace().put(i + 1, linkProps.getNumberOfOccurencesAtGivenPlace().get(i + 1) + 1);
						linkProps.getEnginesHavingAtGivenPlace().put(i + 1, linkProps.getEnginesHavingAtGivenPlace().get(i + 1) + ", " + s);
					}
					else
					{
						linkProps.getNumberOfOccurencesAtGivenPlace().put(i + 1, 1);
						linkProps.getEnginesHavingAtGivenPlace().put(i + 1, s);
					}
				}
			}
		}
		
		for(int i = 0; i < tags.size(); i++)
		{
			HTMLTagA tagA = tags.get(i);
			LinkProperties props = properties.get(i);
			if(props.getNumberOfOccurences() > 1)
			{
				List<Integer> samePlaces = findNumberOfSamePlaces(tagA, results);
				Collections.sort(samePlaces);
				
				int maxOccurences = samePlaces.get(samePlaces.size() - 1);
				
				props.setNumberOfSameplaces(maxOccurences);
				props.setAveragePosition(props.getAveragePosition() / props.getNumberOfOccurences().floatValue());
			}
		}
		
	/*	for(String s : results.keySet())
		{
			List<HTMLTagA> links = results.get(s);
			for(int i = 0; i < links.size(); i++)
			{
				HTMLTagA tagA = links.get(i);
				int index = tags.indexOf(tagA);
				LinkProperties props = properties.get(index);
				if(props.getNumberOfOccurences() > 1)
				{
					List<Integer> samePlaces = findNumberOfSamePlaces(tagA, results);
					Collections.sort(samePlaces);
					
					int maxOccurences = samePlaces.get(samePlaces.size() - 1);
					
					props.setNumberOfSameplaces(maxOccurences);
					props.setAveragePosition(props.getAveragePosition() / props.getNumberOfOccurences().floatValue());
				}
			}
		}*/
		
		for(int i = 0; i < tags.size(); i++)
		{
			linkProperties.put(tags.get(i), properties.get(i));
		}
		
		return linkProperties;
	}
	
	private double calculateAverage(Map<String, List<HTMLTagA>> results)
	{
		double average = 0.0;
		for(String s : results.keySet())
		{
			for(String st : results.keySet())
			{
				if(s.equals(st))
					continue;
				List<HTMLTagA> list1 = results.get(s);
				List<HTMLTagA> list2 = results.get(st);
				
				average += AlgorithmHelper.levensteinDistance(list1, list2);
			}
		}
		
		average /= (results.size() * results.size());
		return average;
	}
	
	private double calculateVariance(Map<String, List<HTMLTagA>> results)
	{
		List<HTMLTagA> averagedSet = new ArrayList<HTMLTagA>();
		
		for(String s : results.keySet())
		{
			List<HTMLTagA> links = results.get(s);
			for(int i = 0; i < links.size(); i++)
			{
				HTMLTagA tagA = (HTMLTagA)links.get(i).clone();
				
				if(!averagedSet.contains(tagA))
					averagedSet.add(tagA);
			}
		}
		
		int setSize = 0;
		
		for(HTMLTagA tagA : averagedSet)
		{
			List<Float> positions = new ArrayList<Float>();
			for(Map.Entry<String, List<HTMLTagA>> entry : results.entrySet())
			{
				List<HTMLTagA> links = entry.getValue();
				
				setSize = links.size();
				
				Integer position = links.indexOf(tagA);
				
				if(position == -1)
					position = links.size();
				positions.add(position.floatValue());
			}
			tagA.setRank(AlgorithmHelper.calculateAverage(positions));
		}
		
		Collections.sort(averagedSet, new Comparator<HTMLTagA>()
				{

					@Override
					public int compare(HTMLTagA o1, HTMLTagA o2)
					{
						return o1.getRank().compareTo(o2.getRank());
					}
			
				});
		
		List<HTMLTagA> tempList = new ArrayList<HTMLTagA>();
		
		for(int i = 0; i < setSize; i++)
		{
			tempList.add(averagedSet.get(i));
		}
		
		averagedSet = tempList;
		
		double deviationSum = 0;
		
		for(Map.Entry<String, List<HTMLTagA>> entry : results.entrySet())
		{
			List<HTMLTagA> list = entry.getValue();
			
			deviationSum += AlgorithmHelper.levensteinDistance(list, averagedSet);
		}
		
		deviationSum /= results.size();
		
		return deviationSum;
	}
	
	private void fillEngineSheet(String engineName, List<HTMLTagA> links, Map<String, List<HTMLTagA>> results)
	{
		HSSFFont headerFont = workbook.createFont();
		headerFont.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
		headerFont.setFontName(HSSFFont.FONT_ARIAL);
		headerFont.setFontHeightInPoints((short)10);
		headerFont.setColor(HSSFFont.COLOR_NORMAL);
		
		HSSFCellStyle greenHeaderStyle = workbook.createCellStyle();
		greenHeaderStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		greenHeaderStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		greenHeaderStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);
		greenHeaderStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);
		greenHeaderStyle.setFillForegroundColor(HSSFColor.GREEN.index);
		greenHeaderStyle.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		greenHeaderStyle.setFont(headerFont);
		greenHeaderStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
		greenHeaderStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
		
		HSSFCellStyle yellowHeaderStyle = workbook.createCellStyle();
		yellowHeaderStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		yellowHeaderStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		yellowHeaderStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);
		yellowHeaderStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);
		yellowHeaderStyle.setFillForegroundColor(HSSFColor.YELLOW.index);
		yellowHeaderStyle.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		yellowHeaderStyle.setFont(headerFont);
		yellowHeaderStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
		yellowHeaderStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
		
		HSSFSheet sheet = workbook.getSheet(engineName);
		
		HSSFRow row0 = workbook.getSheet(engineName).createRow(0);
		HSSFRow row1 = workbook.getSheet(engineName).createRow(1);
		HSSFRow row2 = workbook.getSheet(engineName).createRow(2);
		
		
		HSSFCell engineNameHeader = row0.createCell((short)0);
		engineNameHeader.setCellValue(new HSSFRichTextString(engineName));
		engineNameHeader.setCellStyle(greenHeaderStyle);
		HSSFCell engineNameHeader2 = row0.createCell((short)1);
		engineNameHeader2.setCellStyle(greenHeaderStyle);
		
		sheet.addMergedRegion(new Region(0, (short)0, 0, (short)1));
		
		Map<String, Integer> distances = new TreeMap<String, Integer>();
		
		Float average = 0.0f;
		
		for(Map.Entry<String, List<HTMLTagA>> entry : results.entrySet())
		{
			if(entry.getKey().equals(engineName))
				continue;
			List<HTMLTagA> links2 = entry.getValue();
			
			int distance = AlgorithmHelper.levensteinDistance(links, links2);
			
			distances.put(entry.getKey(), distance);
		
			average += distance;
			
		}
		
		average /= (float)distances.size();
		
		Float stdDev = 0.0f;
		
		for(Map.Entry<String, Integer> entry : distances.entrySet())
		{
			stdDev += Math.pow((double)entry.getValue() - average.doubleValue(), 2.0);
		}
		
		stdDev /= (float)distances.size();
		
		stdDev = (float)Math.sqrt(stdDev.doubleValue());
		
		HSSFCell averageHeader = row1.createCell((short)0);
		averageHeader.setCellValue(new HSSFRichTextString("Average distance"));
		averageHeader.setCellStyle(greenHeaderStyle);
		HSSFCell averageCell = row1.createCell((short)1);
		averageCell.setCellValue(average);
		
		HSSFCell stdDevHeader = row2.createCell((short)0);
		stdDevHeader.setCellValue(new HSSFRichTextString("Standard deviation"));
		stdDevHeader.setCellStyle(greenHeaderStyle);
		HSSFCell stdDevCell = row2.createCell((short)1);
		stdDevCell.setCellValue(stdDev);
		
		int row = 4;
		
		for(Map.Entry<String, Integer> entry : distances.entrySet())
		{
			HSSFCell engineHeader = sheet.createRow(row).createCell((short)0);
			engineHeader.setCellStyle(yellowHeaderStyle);
			engineHeader.setCellValue(new HSSFRichTextString("Distance to " + entry.getKey()));
			
			HSSFCell distanceCell = sheet.createRow(row).createCell((short)1);
			distanceCell.setCellValue(entry.getValue());
			row++;
		}
		
		row++;
		
		HSSFCell linksListHeader = sheet.createRow(row).createCell((short)0);
		linksListHeader.setCellValue(new HSSFRichTextString("Links"));
		linksListHeader.setCellStyle(greenHeaderStyle);
		
		row++;
		
		for(int i = 0; i < links.size(); i++)
		{
			HSSFRow rowi = sheet.createRow(row);
			
			HSSFCell cell = rowi.createCell((short)0);
			
			cell.setCellValue(new HSSFRichTextString(links.get(i).getHREF()));
			row++;
		}
		
		for(short i = 0; i < 2; i++)
		{
			sheet.setColumnWidth(i, (short)8000);
		}
		
	}
	
	private void createSummarySheetHeaders()
	{
		
		HSSFFont headerFont = workbook.createFont();
		headerFont.setFontName(HSSFFont.FONT_ARIAL);
		headerFont.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
		headerFont.setFontHeightInPoints((short)10);
		headerFont.setColor(HSSFColor.BLACK.index);
		
		HSSFCellStyle headerStyle = workbook.createCellStyle();
		headerStyle.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		headerStyle.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		headerStyle.setBorderTop(HSSFCellStyle.BORDER_THIN);
		headerStyle.setBorderRight(HSSFCellStyle.BORDER_THIN);
		headerStyle.setFillForegroundColor(HSSFColor.GREEN.index);
		headerStyle.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		headerStyle.setFont(headerFont);
		headerStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
		headerStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
		
		HSSFRow row0 = workbook.getSheetAt(0).createRow(0);
		HSSFRow row1 = workbook.getSheetAt(0).createRow(1);
		HSSFRow row2 = workbook.getSheetAt(0).createRow(2);
		
		HSSFCell averageHeader = row0.createCell((short)0);
		HSSFCell stdDevHeader = row0.createCell((short)1);
		
		averageHeader.setCellValue(new HSSFRichTextString("Average distance"));
		averageHeader.setCellStyle(headerStyle);
		row1.createCell((short)0).setCellStyle(headerStyle);
		row2.createCell((short)0).setCellStyle(headerStyle);
		stdDevHeader.setCellValue(new HSSFRichTextString("Standard deviation"));
		stdDevHeader.setCellStyle(headerStyle);
		row1.createCell((short)1).setCellStyle(headerStyle);
		row2.createCell((short)1).setCellStyle(headerStyle);
		
		workbook.getSheetAt(0).addMergedRegion(new Region(0, (short)0, 2, (short)0));
		workbook.getSheetAt(0).addMergedRegion(new Region(0, (short)1, 2, (short)1));
		
		HSSFRow row5 = workbook.getSheetAt(0).createRow(5);
		HSSFRow row6 = workbook.getSheetAt(0).createRow(6);
		HSSFRow row7 = workbook.getSheetAt(0).createRow(7);
		
		HSSFCell linksListHeader = row5.createCell((short)0);
		HSSFCell engineHeader = row5.createCell((short)1);
		HSSFCell numberOfOccurencesHeader = row5.createCell((short)2);
		HSSFCell averagePlaceHeader = row5.createCell((short)3);
		HSSFCell minimalPlaceHeader = row5.createCell((short)4);
		HSSFCell maximalPlaceHeader = row5.createCell((short)5);
		HSSFCell numberOfSamePlaceOccurencesHeader = row5.createCell((short)6);
		HSSFCell onPlaceOccurenceHeader = row5.createCell((short)7);
		
		linksListHeader.setCellValue(new HSSFRichTextString("Links"));
		linksListHeader.setCellStyle(headerStyle);
		row6.createCell((short)0).setCellStyle(headerStyle);
		row7.createCell((short)0).setCellStyle(headerStyle);
		engineHeader.setCellValue(new HSSFRichTextString("Search Engines"));
		engineHeader.setCellStyle(headerStyle);
		row6.createCell((short)1).setCellStyle(headerStyle);
		row7.createCell((short)1).setCellStyle(headerStyle);
		numberOfOccurencesHeader.setCellValue(new HSSFRichTextString("Number of occurences in different engines"));
		numberOfOccurencesHeader.setCellStyle(headerStyle);
		row6.createCell((short)2).setCellStyle(headerStyle);
		row7.createCell((short)2).setCellStyle(headerStyle);
		averagePlaceHeader.setCellValue(new HSSFRichTextString("Average place"));
		averagePlaceHeader.setCellStyle(headerStyle);
		row6.createCell((short)3).setCellStyle(headerStyle);
		row7.createCell((short)3).setCellStyle(headerStyle);
		minimalPlaceHeader.setCellValue(new HSSFRichTextString("Minimal place"));
		minimalPlaceHeader.setCellStyle(headerStyle);
		row6.createCell((short)4).setCellStyle(headerStyle);
		row7.createCell((short)4).setCellStyle(headerStyle);
		maximalPlaceHeader.setCellValue(new HSSFRichTextString("Maximal place"));
		maximalPlaceHeader.setCellStyle(headerStyle);
		row6.createCell((short)5).setCellStyle(headerStyle);
		row7.createCell((short)5).setCellStyle(headerStyle);
		numberOfSamePlaceOccurencesHeader.setCellValue(new HSSFRichTextString("Number of occurences on the same place"));
		numberOfSamePlaceOccurencesHeader.setCellStyle(headerStyle);
		row6.createCell((short)6).setCellStyle(headerStyle);
		row7.createCell((short)6).setCellStyle(headerStyle);
		onPlaceOccurenceHeader.setCellValue(new HSSFRichTextString("Number of occurences at given place"));
		onPlaceOccurenceHeader.setCellStyle(headerStyle);
		row6.createCell((short)7).setCellStyle(headerStyle);
		row7.createCell((short)7).setCellStyle(headerStyle);
		
	
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)0, 7, (short)0));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)1, 7, (short)1));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)2, 7, (short)2));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)3, 7, (short)3));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)4, 7, (short)4));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)5, 7, (short)5));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)6, 7, (short)6));
		workbook.getSheetAt(0).addMergedRegion(new Region(5, (short)7, 7, (short)7));
	}
	
	private List<Integer> findNumberOfSamePlaces(HTMLTagA tagA, Map<String, List<HTMLTagA>> results)
	{
		List<Integer> numberOfSamePlaces = new ArrayList<Integer>();
		
		
		Map<String, Integer> linkIndexes = new HashMap<String, Integer>();
		Map<Integer, Integer> indexOccurences = new HashMap<Integer, Integer>();
		
		for(String s : results.keySet())
		{
			List<HTMLTagA> links = results.get(s);
			linkIndexes.put(s, links.indexOf(tagA));
		}
		
		for(String s : results.keySet())
		{
			Integer linkIndex = linkIndexes.get(s);
			if(!indexOccurences.containsKey(linkIndex))
			{
				indexOccurences.put(linkIndex, 1);
			}
			else
			{
				indexOccurences.put(linkIndex, indexOccurences.get(linkIndex) + 1);
			}
		}
		
		for(Integer index : indexOccurences.keySet())
		{
			if(index == -1)
				continue;
			numberOfSamePlaces.add(indexOccurences.get(index));
		}
		
		return numberOfSamePlaces;
	}
	
	private class LinkProperties
	{
		private String searchEngine;
		private Integer numberOfOccurences;
		private Integer numberOfSameplaces;
		private Map<Integer, Integer> numberOfOccurencesAtGivenPlace;
		private Map<Integer, String> enginesHavingAtGivenPlace;
		private Float averagePosition;
		private Integer minimalPosition;
		private Integer maximalPosition;
		
		public Integer getMinimalPosition()
		{
			return minimalPosition;
		}
		public void setMinimalPosition(Integer minimalPosition)
		{
			this.minimalPosition = minimalPosition;
		}
		public Integer getMaximalPosition()
		{
			return maximalPosition;
		}
		public void setMaximalPosition(Integer maximalPosition)
		{
			this.maximalPosition = maximalPosition;
		}
		public Float getAveragePosition()
		{
			return averagePosition;
		}
		public void setAveragePosition(Float averagePosition)
		{
			this.averagePosition = averagePosition;
		}
		public Map<Integer, String> getEnginesHavingAtGivenPlace()
		{
			if(enginesHavingAtGivenPlace == null)
				enginesHavingAtGivenPlace = new TreeMap<Integer, String>();
			return enginesHavingAtGivenPlace;
		}
		public void setEnginesHavingAtGivenPlace(
				Map<Integer, String> enginesHavingAtGivenPlace)
		{
			this.enginesHavingAtGivenPlace = enginesHavingAtGivenPlace;
		}
		public Map<Integer, Integer> getNumberOfOccurencesAtGivenPlace()
		{
			if(numberOfOccurencesAtGivenPlace == null)
				numberOfOccurencesAtGivenPlace = new TreeMap<Integer, Integer>();
			return numberOfOccurencesAtGivenPlace;
		}
		public void setNumberOfOccurencesAtGivenPlace(
				Map<Integer, Integer> numberOfOccurencesAtGivenPlace)
		{
			this.numberOfOccurencesAtGivenPlace = numberOfOccurencesAtGivenPlace;
		}
		public String getSearchEngine()
		{
			return searchEngine;
		}
		public void setSearchEngine(String searchEngine)
		{
			this.searchEngine = searchEngine;
		}
		public Integer getNumberOfOccurences()
		{
			return numberOfOccurences;
		}
		public void setNumberOfOccurences(Integer numberOfOccurences)
		{
			this.numberOfOccurences = numberOfOccurences;
		}
		public Integer getNumberOfSameplaces()
		{
			return numberOfSameplaces;
		}
		public void setNumberOfSameplaces(Integer numberOfSameplaces)
		{
			this.numberOfSameplaces = numberOfSameplaces;
		}
	}
}
