package pw.stadnik.searcherclient;

import jade.wrapper.gateway.JadeGateway;

import java.io.IOException;
import java.util.ArrayList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import pw.stadnik.common.HTMLTagA;
import pw.stadnik.common.SearchParams;

/**
 * 
 * GatewayServlet serves as a backend of the client application.
 * Every client request is processed here.
 * 
 * @author Jakub Stadnik
 *
 */
public class GatewayServlet extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet 
{
	static final long serialVersionUID = 1L;
	/**
	 * Field is set to number of miliseconds
	 * for which VM should put the thread to
	 * sleep before checking the availablity
	 * of JadeGateway.
	 */
	private static final long threadSleepTime = 5000;
    /* (non-Java-doc)
	 * @see javax.servlet.http.HttpServlet#HttpServlet()
	 */
	public GatewayServlet() 
	{
		super();
	}   	 	
	
	/* (non-Java-doc)
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
	{
		String stepParam = request.getParameter("step");
		Integer step = Integer.parseInt(stepParam);
		switch(step)
		{
			case 1:
				postFirstStep(request, response);
				break;
			case 2:
				postSecondStep(request, response);
				break;
			default:
				break;
		}
		
	}   	  	  
	
	/**
	 * Method processes initial client request, that is
	 * query and algorithm that was input. Then it starts
	 * SearcherClientAgent which in turn sends client 
	 * parameters to the main application.
	 * 
	 * @param request a Http request
	 * @param response a Http response
	 * @throws ServletException if servlet error occurs
	 * @throws IOException if I/O error occurs
	 */
	private void postFirstStep(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
	{
		while(JadeGateway.isGatewayActive())
		{
			try
			{
				Thread.sleep(threadSleepTime);
			}
			catch (Exception e) 
			{
				e.printStackTrace();
			}
		}
		
		JadeGateway.init("pw.stadnik.searcherclient.SearcherClientAgent", null);
		String query = request.getParameter("query");
		
		if(query == null || query.equals(""))
		{
			response.getWriter().println("Please input proper query");
			return;
		}
		String algorithm = request.getParameter("algorithm");
		if(algorithm == null || algorithm.equals(""))
		{
			response.getWriter().println("Select proper algorithm");
			return;
		}
		
		String[] splittedQuery = query.split(" ");
		ArrayList<String> phrases = new ArrayList<String>();
		for(int i = 0; i < splittedQuery.length; i++)
		{
			phrases.add(splittedQuery[i]);
		}
		
		SearchParams sp = new SearchParams();
		sp.setAlgorithm(algorithm);
		sp.setPhrases(phrases);
		
		HttpSession session = request.getSession();
		session.setAttribute("searchParams", sp);
		try
		{
			JadeGateway.execute(session);
			JadeGateway.shutdown();
			response.sendRedirect("results.jsp");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	/**
	 * Method processes request containing feedback
	 * that user sends to the main application.
	 * After processing it starts FeedbackAgent
	 * which sends the feedback to the main application.
	 * 
	 * @param request a Http request
	 * @param response a Http response
	 * @throws ServletException if servlet error occurs
	 * @throws IOException if I/O error occurs
	 */
	private void postSecondStep(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
	{
		while(JadeGateway.isGatewayActive())
		{
			try
			{
				Thread.sleep(threadSleepTime);
			}
			catch (Exception e) 
			{
				e.printStackTrace();
			}
		}
		JadeGateway.init("pw.stadnik.searcherclient.FeedbackAgent", null);
		String linkName = request.getParameter("linkName");
		String href = request.getParameter("href");
		
		HTMLTagA feedbackLink = new HTMLTagA();
		feedbackLink.setHREF(href);
		feedbackLink.setLinkName(linkName);
		
		HttpSession session = request.getSession();
		
		session.setAttribute("feedbackLink", feedbackLink);
		session.setAttribute("feedbackRequired", new Boolean(true));
		
		try
		{
			JadeGateway.execute(session);
			JadeGateway.shutdown();
			session.setAttribute("feedbackRequired", new Boolean(false));
			response.sendRedirect("results.jsp");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}