package pw.stadnik.searcherclient;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Random;

import javax.servlet.http.HttpSession;

import pw.stadnik.common.HTMLTagA;
import pw.stadnik.common.SearchParams;
import jade.core.AID;
import jade.core.ProfileImpl;
import jade.core.Runtime;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import jade.wrapper.gateway.GatewayAgent;

/**
 * An agent that forwards user request (query, algorithm)
 * to the main application. It also creates a new ManagerAgent
 * if necessary and receives answers from the main system.
 * 
 * @author Jakub Stadnik
 *
 */
public class SearcherClientAgent extends GatewayAgent
{

	/**
	 * Serial version UID. It is used when 
	 * serializing a particular object of this class.
	 */
	private static final long serialVersionUID = 762612104265847825L;
	/**
	 * HTTP session of the user.
	 */
	private HttpSession session = null;
	
	/** 
	 * Processes object session. SearchParams object is extracted 
	 * forwarded to the main system. Also (if necessary) the ManagerAgent is created.
	 **/
	@Override
	protected void processCommand(Object arg0)
	{	
		try
		{
			if(arg0 instanceof HttpSession)
			{
				session = (HttpSession)arg0;
				SearchParams searchParams = (SearchParams)session.getAttribute("searchParams");
				
				AID managerAgentAID = null;
				if((managerAgentAID = (AID)session.getAttribute("managerAgentAID")) == null)
				{
					Runtime r = Runtime.instance();
					AgentContainer container = r.createAgentContainer(new ProfileImpl());
					String managerAgentName = "ManagerAgent_";
					Random random = new Random(System.currentTimeMillis());
					for(int i = 0; i < 10; i++)
					{
						managerAgentName += String.valueOf(random.nextInt(10));
					}
					AgentController c = container.createNewAgent(managerAgentName, "pw.stadnik.searcher.agent.ManagerAgent", new Object[]{managerAgentName});
					c.start();
					
					session.setAttribute("managerAgentController", c);
					
					managerAgentAID = new AID(managerAgentName, AID.ISLOCALNAME);
		
					session.setAttribute("managerAgentAID", managerAgentAID);
					
				}
				
				AgentController c = (AgentController)session.getAttribute("managerAgentController");
				
				System.out.println(c.getState().toString());
				
				ACLMessage paramMessage = new ACLMessage(ACLMessage.REQUEST);
				paramMessage.setContentObject(searchParams);
				paramMessage.addReceiver(managerAgentAID);
				System.out.println(managerAgentAID);
				
				
				send(paramMessage);
				
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		addBehaviour(new Behaviour(this)
		{
	
			/**
			 * 
			 */
			private static final long serialVersionUID = 6237029653653881282L;
			private boolean stop = false;
			@Override
			public void action()
			{	
				obtainAnswer();
				stop = true;
			}
			@Override
			public boolean done()
			{
				return stop;
			}
		});
	}
	
	/**
	 * Method puts the agent into waiting state.
	 * Agent waits for the answer list from the
	 * main system.
	 */
	@SuppressWarnings("unchecked")
	private void obtainAnswer()
	{
		ACLMessage message = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
		try
		{
			if(message != null)
			{
				Serializable s = message.getContentObject();
				if(s instanceof String)
				{
					String fileName = (String)s;
					session.setAttribute("fileName", fileName);
					releaseCommand(session);
				}
				else if(s instanceof ArrayList)
				{
					//ArrayList<HTMLTagA> answer = (ArrayList<HTMLTagA>);
					session.setAttribute("answers", message.getContentObject());
					message = blockingReceive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
					session.setAttribute("feedbackRequired", message.getContentObject());
					releaseCommand(session);
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
